@@ -115,7 +115,7 @@ class JupyterWidget(IPythonWidget):
     #---------------------------------------------------------------------------
 
     def __init__(self, *args, **kw):
-        super(JupyterWidget, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
 
         # JupyterWidget protected variables.
         self._payload_handlers = {
@@ -135,6 +135,7 @@ class JupyterWidget(IPythonWidget):
 
         # Initialize language name.
         self.language_name = None
+        self._prompt_requested = False
 
     #---------------------------------------------------------------------------
     # 'BaseFrontendMixin' abstract interface
@@ -183,6 +184,7 @@ class JupyterWidget(IPythonWidget):
         msg_id = msg['parent_header'].get('msg_id')
         info = self._request_info['execute'].get(msg_id)
         if info and info.kind == 'prompt':
+            self._prompt_requested = False
             content = msg['content']
             if content['status'] == 'aborted':
                 self._show_interpreter_prompt()
@@ -191,7 +193,7 @@ class JupyterWidget(IPythonWidget):
                 self._show_interpreter_prompt(number)
             self._request_info['execute'].pop(msg_id)
         else:
-            super(JupyterWidget, self)._handle_execute_reply(msg)
+            super()._handle_execute_reply(msg)
 
     def _handle_history_reply(self, msg):
         """ Handle history tail replies, which are only supported
@@ -218,7 +220,7 @@ class JupyterWidget(IPythonWidget):
         history_items = content['history']
         self.log.debug("Received history reply with %i entries", len(history_items))
         items = []
-        last_cell = u""
+        last_cell = ""
         for _, _, cell in history_items:
             cell = cell.rstrip()
             if cell != last_cell:
@@ -226,11 +228,11 @@ class JupyterWidget(IPythonWidget):
                 last_cell = cell
         self._set_history(items)
 
-    def _insert_other_input(self, cursor, content):
+    def _insert_other_input(self, cursor, content, remote=True):
         """Insert function for input from other frontends"""
         n = content.get('execution_count', 0)
-        prompt = self._make_in_prompt(n, remote=True)
-        cont_prompt = self._make_continuation_prompt(self._prompt, remote=True)
+        prompt = self._make_in_prompt(n, remote=remote)
+        cont_prompt = self._make_continuation_prompt(self._prompt, remote=remote)
         cursor.insertText('\n')
         for i, line in enumerate(content['code'].strip().split('\n')):
             if i == 0:
@@ -246,7 +248,12 @@ class JupyterWidget(IPythonWidget):
         """Handle an execute_input message"""
         self.log.debug("execute_input: %s", msg.get('content', ''))
         if self.include_output(msg):
-            self._append_custom(self._insert_other_input, msg['content'], before_prompt=True)
+            self._append_custom(
+                self._insert_other_input, msg['content'], before_prompt=True)
+        elif not self._prompt:
+            self._append_custom(
+                self._insert_other_input, msg['content'],
+                before_prompt=True, remote=False)
 
     def _handle_execute_result(self, msg):
         """Handle an execute_result message"""
@@ -287,7 +294,7 @@ class JupyterWidget(IPythonWidget):
                 text = data['text/plain']
                 self._append_plain_text(text, True)
             # This newline seems to be needed for text and html output.
-            self._append_plain_text(u'\n', True)
+            self._append_plain_text('\n', True)
 
     def _handle_kernel_info_reply(self, rep):
         """Handle kernel info replies."""
@@ -312,7 +319,7 @@ class JupyterWidget(IPythonWidget):
         if self._starting:
             # finish handling started channels
             self._starting = False
-            super(JupyterWidget, self)._started_channels()
+            super()._started_channels()
 
     def _started_channels(self):
         """Make a history request"""
@@ -366,8 +373,12 @@ class JupyterWidget(IPythonWidget):
         """
         # If a number was not specified, make a prompt number request.
         if number is None:
+            if self._prompt_requested:
+                # Already asked for prompt, avoid multiple prompts.
+                return
+            self._prompt_requested = True
             msg_id = self.kernel_client.execute('', silent=True)
-            info = self._ExecutionRequest(msg_id, 'prompt')
+            info = self._ExecutionRequest(msg_id, 'prompt', False)
             self._request_info['execute'][msg_id] = info
             return
 
@@ -385,6 +396,9 @@ class JupyterWidget(IPythonWidget):
 
     def _update_prompt(self, new_prompt_number):
         """Replace the last displayed prompt with a new one."""
+        if self._previous_prompt_obj is None:
+            return
+
         block = self._previous_prompt_obj.block
 
         # Make sure the prompt block has not been erased.
@@ -604,4 +618,4 @@ class IPythonWidget(JupyterWidget):
     def __init__(self, *a, **kw):
         warn("IPythonWidget is deprecated; use JupyterWidget",
              DeprecationWarning)
-        super(IPythonWidget, self).__init__(*a, **kw)
+        super().__init__(*a, **kw)
